@@ -87,6 +87,28 @@ class EcoNetApiInterface:
         await this_class._authenticate({"email": email, "password": password})
         return this_class
 
+    def check_mode_enum(self, equip, enumtext=False):
+        # Fix enumeration of Emergency Heat in Thermostat, maybe others?
+        if "@MODE" in equip and isinstance(equip["@MODE"], Dict):
+            if 'constraints' in equip["@MODE"]:
+                enumtext = equip["@MODE"]['constraints']['enumText']
+            if enumtext:
+                value = equip["@MODE"]['value']
+                status = equip["@MODE"]['status']
+                if (value != enumtext.index(status)):
+                    _LOGGER.debug("Enum value mismatch: "
+                                  f"{enumtext[value]} != "
+                                  f"{equip["@MODE"]['status']}")
+                    equip["@MODE"]['value'] = enumtext.index(status)
+        return equip, enumtext
+
+    def check_update_enum(self, equip, update):
+        # Update messages only have the status and value, so get the enumtext
+        equip, enumtext = self.check_mode_enum(equip)
+        # Fix the update
+        update, __ = self.check_mode_enum(update, enumtext)
+        return equip, update
+
     def subscribe(self):
         """Subscribe to the MQTT updates"""
         if not self._equipment:
@@ -132,7 +154,7 @@ class EcoNetApiInterface:
         )
 
     def unsubscribe(self) -> None:
-        self._mqtt_client.loop_stop(force=True)
+        self._mqtt_client.loop_stop()
 
     def _get_client_id(self) -> str:
         time_string = str(time.time()).replace(".", "")[:13]
@@ -144,6 +166,12 @@ class EcoNetApiInterface:
         for _location in _locations:
             # They spelled it wrong...
             for _equip in _location.get("equiptments"):
+                # Early exit if server returned error code
+                if "error" in _equip:
+                    _LOGGER.debug("EcoNet equipment error message"
+                                  f": {_equip.get('error')}")
+                    continue
+                _equip, __ = self.check_mode_enum(_equip)
                 _equip_obj: Equipment = None
                 if (
                     Equipment._coerce_type_from_string(_equip.get("device_type"))
@@ -170,6 +198,7 @@ class EcoNetApiInterface:
                 serial = _equip.get("serial_number")
                 equipment = self._equipment.get(serial)
                 if equipment:
+                    _equip, __ = self.check_mode_enum(_equip)
                     equipment.update_equipment_info(_equip)
 
     async def get_equipment_by_type(self, equipment_type: List) -> Dict:
@@ -196,7 +225,7 @@ class EcoNetApiInterface:
             ) as resp:
                 if resp.status == 200:
                     _json = await resp.json()
-                    _LOGGER.debug(_json)
+                    _LOGGER.debug(json.dumps(_json, indent=2))
                     if _json.get("success"):
                         self._locations = _json["results"]["locations"]
                         return self._locations
@@ -222,7 +251,7 @@ class EcoNetApiInterface:
             ) as resp:
                 if resp.status == 200:
                     _json = await resp.json()
-                    _LOGGER.debug(_json)
+                    _LOGGER.debug(json.dumps(_json, indent=2))
                     if _json.get("success"):
                         return _json
 
@@ -243,7 +272,7 @@ class EcoNetApiInterface:
           ) as resp:
               if resp.status == 200:
                   _json = await resp.json()
-                  _LOGGER.debug(_json)
+                  _LOGGER.debug(json.dumps(_json, indent=2))
                   if _json.get("options")["success"]:
                       self._user_token = _json.get("user_token")
                       self._account_id = _json.get("options").get("account_id")
@@ -276,6 +305,7 @@ class EcoNetApiInterface:
             key = _serial
             _equipment = self._equipment.get(key)
             if _equipment is not None:
+                _equipment._equipment_info, unpacked_json = self.check_update_enum(_equipment._equipment_info, unpacked_json)
                 _equipment.update_equipment_info(unpacked_json)
             # Nasty hack to push signal updates to the device it belongs to
             elif "@SIGNAL" in str(unpacked_json):
